@@ -1,9 +1,13 @@
 """MQTT base entity module."""
+import asyncio
 import json
 import logging
-from typing import Any, Dict, Optional, TypedDict, Union
+from collections.abc import AsyncGenerator, Callable
+from contextlib import AsyncExitStack
+from typing import Any, TypedDict
 
-import paho.mqtt.client as mqtt
+import asyncio_mqtt as mqtt
+import paho.mqtt.client as paho
 
 from mqtt_hass_base.error import MQTTHassBaseError
 
@@ -12,7 +16,9 @@ from mqtt_hass_base.error import MQTTHassBaseError
 
 
 # TODO: python 3.11 => remove total and uncomment NotRequired
-class EntitySettingsType(TypedDict, total=False):
+class EntitySettingsType(
+    TypedDict, total=False
+):  # pylint: disable=too-few-public-methods
     """Base entity settings dict format."""
 
     object_id: str
@@ -23,26 +29,30 @@ class MqttEntity:
     """MQTT base entity class."""
 
     _component: str = ""
-    _subscriptions: Dict[str, str] = {}
-    _device_payload: Dict[str, Any] = {}
+    _subscriptions: dict[str, Callable[..., Any]] = {}
+    _device_payload: dict[str, Any] = {}
 
     def __init__(
         self,
         name: str,
         unique_id: str,
+        mqtt_client: mqtt.Client,
         mqtt_discovery_root_topic: str,
         mqtt_data_root_topic: str,
         logger: logging.Logger,
-        device_payload: Optional[Dict[str, str]] = None,
-        subscriptions: Optional[Dict[str, str]] = None,
-        object_id: Optional[str] = None,
+        device_payload: dict[str, str] | None = None,
+        subscriptions: dict[str, Callable[..., Any]] | None = None,
+        object_id: str | None = None,
+        entity_category: str | None = None,
     ):
         """Create a new MQTT entity."""
         self._name: str = name
+        self._mqtt_client = mqtt_client
         self._mqtt_discovery_root_topic = mqtt_discovery_root_topic
         self._mqtt_data_root_topic = mqtt_data_root_topic
         self._logger = logger.getChild(name)
         self._object_id = object_id
+        self._entity_category = entity_category
         self._unique_id = unique_id
 
         if device_payload is not None:
@@ -86,10 +96,15 @@ class MqttEntity:
         return self._name
 
     @property
-    def object_id(self) -> Optional[str]:
+    def object_id(self) -> str | None:
         """Get the object_id of the entity."""
         return self._object_id
 
+    @property
+    def entity_category(self) -> str | None:
+        """Get the entity_category of the entity."""
+        return self._entity_category
+
     @property
     def unique_id(self) -> str:
         """Get the unique_id of the entity."""
@@ -140,58 +155,66 @@ class MqttEntity:
             )
         ).lower()
 
-    def send_available(self) -> None:
+    async def _on_messages(
+        self,
+        messages: AsyncGenerator[paho.MQTTMessage, None],
+        on_message_callback: Callable[..., Any],
+    ) -> None:
+        """on_command MQTT callback."""
+        try:
+            async for message in messages:
+                await on_message_callback(message)
+        except mqtt.MqttError as exp:
+            self.logger.warning(exp)
+
+    async def send_available(self) -> None:
         """Set the availability to ON."""
-        self._mqtt_client.publish(
+        await self._mqtt_client.publish(
             topic=self.availability_topic, retain=True, payload="online"
         )
 
-    def send_not_available(self) -> None:
+    async def send_not_available(self) -> None:
         """Set the availability to OFF."""
-        self._mqtt_client.publish(
+        await self._mqtt_client.publish(
             topic=self.availability_topic, retain=True, payload="offline"
         )
 
-    def register(self) -> None:
+    async def register(self) -> None:
         """Register the current entity to Hass.
 
         Using the MQTT discovery feature of Home Assistant.
         """
         raise NotImplementedError
 
-    def subscribe(self) -> None:
-        """Subscribe to the MQTT topics."""
-        for topic_name, callback in self._subscriptions.items():
-            self.logger.info("Subscribing to %s", topic_name)
-            if not hasattr(self, topic_name):
-                msg = f"This entity doesn't have this mqtt input topic {topic_name}"
-                self.logger.error(msg)
-                raise MQTTHassBaseError(msg)
-            topic = getattr(self, topic_name)
-            self._mqtt_client.subscribe(topic)
-            self._mqtt_client.message_callback_add(topic, callback)
+    async def subscribe(self, stack: AsyncExitStack) -> set[asyncio.Task[None]]:
+        """Subscribe to all mqtt topics needed."""
+        raise NotImplementedError
 
-    def unregister(self) -> None:
+    async def unregister(self) -> None:
         """Unregister the current entity to Hass.
 
         Using the MQTT discovery feature of Home Assistant.
         """
         self.logger.info("Deleting %s: %s", self.config_topic, json.dumps({}))
-        self._mqtt_client.publish(topic=self.config_topic, retain=False, payload=None)
+        await self._mqtt_client.publish(
+            topic=self.config_topic, retain=False, payload=None
+        )
 
-    def send_state(
+    async def send_state(
         self,
-        state: Union[str, bytes, float, int],
-        attributes: Optional[Dict[str, Any]] = None,
+        state: str | bytes | float | int,
+        attributes: dict[str, Any] | None = None,
     ) -> None:
         """Send current state to MQTT."""
         raise NotImplementedError
 
-    def send_attributes(self, attributes: Dict[str, Any]) -> None:
+    async def send_attributes(self, attributes: dict[str, Any] | None) -> None:
         """Send current attribute dict to MQTT."""
+        if attributes in (None, {}):
+            return
         if not isinstance(attributes, dict):
             raise ValueError("'attributes' argument should be a dict.")
-        self._mqtt_client.publish(
+        await self._mqtt_client.publish(
             topic=self.json_attributes_topic,
             retain=True,
             payload=json.dumps(attributes),
